#ifndef ae_h
#define ae_h

/*
ae.h

Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com
Subcontract/D.O. DDL 04-D-4032-010
DDL Omni Engineering
Prime Contract N00178-04-D-4032, D.O. FD02
NSWCCD Code 653

JHSV Analysis Engine - NSWCCD-65-TR-2006/15

This program performs statistical analysis on SCL binary data files. This program was originally written
to support the JHSV hull monitoring system, but its analysis capabitlities are useful far beyond the scope of
the project.

If you are attempting to modify this code, it is best to start with the main() function in order to uderstand
what the program does from a high-level standpoint first.

This header file contains functions that may be easily used by other programs.
*/

#include<iostream>//for cout and standard i/o
#include<fstream>//for file i/o
#include<vector>//for STL vector
#include<string>//for string
#include<math.h>//for math functions
#include<time.h>//for time functions
using namespace std;//using the standard namespace


//global variables - user defined
static unsigned int fftsize = 2048;//fftsize for PSD segments must be a power of 2: 256, 512, 1024, 2048, 4096, 8192
static unsigned int wavechannel = 0;//defines wave height channel to divide by for rao analysis
static float cutofffrequency = 0;//defines the cutoff frequency for lowpass and highpass filters 0 does not filter
//global variables - single values
static int numchan = 0;//number of real channels in input data file, if virtual it is real+virtual
static int realnumchan = 0;//number of real channels in input data file
static int virtualnumchan = 0;//number of virtual channels in input data file
static float modelscaleratio = 1;//the scale ratio (one to x) of the model, like 1:18 or 1:30
static float scanrate = 0;//scan rate of input data file
static float fullscanrate = 0;//scan rate of input data file with the model scale ratio applied
static int confignumchan = 0;//number of channels in configuration file
static float configscanrate = 0;//scan rate corresponding to configuration file
static double cal = 0;//temporary cal factor from input data file
static float datapt = 0;//temporary data point from input data file
static unsigned int originaldatasize = 0;//original size of the data; number of scans
static unsigned int powerof2 = 0;//used to find the next power of 2 beyond data size
//global variables - time history storage
static vector<double>cals;//vector stores all cal factors from input data file
static vector< vector<float> > data;//vector stores actual time histories of input data file
static vector< vector<float> > datalp;//vector stores actual time histories of input data file lowpass filtered
static vector< vector<float> > datahp;//vector stores actual time histories of input data file highpass filtered
//global variables - mmm analysis
static vector<double> means;//vector stores the mean of each channel of the input data file
static vector<float> maxes;//vector stores the max of each channel of the input data file
static vector<float> mins;//vector stores the min of each channel of the input data file
static vector<double> variances;//vector stores the variance of each channel of the input data file
static vector<double> skews;//vector stores the shewness of each channel of the input data file
static vector<double> kurtosi;//vector stores the kurtosis of each channel of the input data file
static vector<double> meanslp;//vector stores the mean of each channel of the lowpass input data file
static vector<float> maxeslp;//vector stores the max of each channel of the lowpass input data file
static vector<float> minslp;//vector stores the min of each channel of the lowpass input data file
static vector<double> varianceslp;//vector stores the variance of each channel of the lowpass input data file
static vector<double> skewslp;//vector stores the shewness of each channel of the input data file
static vector<double> kurtosilp;//vector stores the kurtosis of each channel of the lowpass input data file
static vector<double> meanshp;//vector stores the mean of each channel of the highpass input data file
static vector<float> maxeshp;//vector stores the max of each channel of the highpass input data file
static vector<float> minshp;//vector stores the min of each channel of the highpass input data file
static vector<double> varianceshp;//vector stores the variance of each channel of the highpass input data file
static vector<double> skewshp;//vector stores the shewness of each channel of the input data file
static vector<double> kurtosihp;//vector stores the kurtosis of each channel of the highpass input data file
//global variables - rainflow analysis
static vector< vector<float> > extrema;//vector stores extrema detected during rainflow analysis
static vector< vector<unsigned int> > extremaindex;//vector stores extrema detected during rainflow analysis
static vector<float> tempextrema;//vector temporarily stores extrema detected during rainflow analysis
static vector<float> tempcycles;//vector temporarily stores cycles counted during rainflow analysis
static vector< vector<float> > rawcycles;//vector stores cycles detected during rainflow analysis
static vector<double> damage;//vector stores damage calculated for the input data file
static int bins=20;//number of bins for the histograms
static vector< vector<float> > fullcyclehistograms;//vector stores histograms of full cycles detected
static vector<double> meanscycles;//vector stores the mean of each channel of the highpass input data file
static vector<float> maxescycles;//vector stores the max of each channel of the highpass input data file
static vector<float> minscycles;//vector stores the min of each channel of the highpass input data file
static vector<double> variancescycles;//vector stores the variance of each channel of the highpass input data file
static vector<double> skewscycles;//vector stores the shewness of each channel of the input data file
static vector<double> kurtosicycles;//vector stores the kurtosis of each channel of the highpass input data file
//global variables - frequency domain analysis
static vector<vector<float> > psds;//vector stores the psd of each channel of the input data file
static vector<float> psdpeakfreqs;//vector stores the frequency of the psd maximum for each channel
static vector<float> psd99pctfreqs;//vector stores the frequency of the psd when 99% of the energy has been reached for each channel
static vector<vector<float> > raos;//vector stores the rao of each channel of the input data file
static float* paddedrealdata = NULL;//used for padded data during fft filtering
static float* paddedimaginarydata = NULL;//used for padded data during fft filtering
//global variables - weibull analysis
class weibullstorage{
public:
	vector<float> amplitudes;//stores the amplitudes for this 2-parameter Weibull analysis
	vector<float> rank;//stores the rank of each amplitude
	double amplitudemean;//mean of the amplitudes
	float amplitudemax;//max of the amplitudes
	float amplitudemin;//min of the amplitudes
	double amplitudevariance;//variance of the amplitudes
	double amplitudeskew;//skew of the amplitudes
	double amplitudekurtosis;//kurtosis of the amplitudes
	double lrbeta;//linear regression beta
	double lryint;//linear regression y intercept
	double lrcharval;//linear regression characteristic value
	double lrcorrelation;//linear regression correlation (r^2)
	double lrcdf99;//expected x at CDF=0.99
	double lrcdf999;//expected x at CDF=0.999
	double lrcdf9999;//expected x at CDF=0.9999
	double mmbeta;//moment method beta
	double mmyint;//moment method y intercept
	double mmcharval;//moment method charactcteristic value
	double mmcdf99;//expected x at CDF=0.99
	double mmcdf999;//expected x at CDF=0.999
	double mmcdf9999;//expected x at CDF=0.9999
	weibullstorage(){
		amplitudemean = 0;
		amplitudemax = 0;
		amplitudemin = 0;
		amplitudevariance = 0;
		amplitudeskew = 0;
		amplitudekurtosis = 0;
		lrbeta = 0;
		lryint = 0;
		lrcharval = 0;
		lrcorrelation = 0;
		lrcdf99 = 0;
		lrcdf999 = 0;
		lrcdf9999 = 0;
		mmbeta = 0;
		mmyint = 0;
		mmcharval = 0;
		mmcdf99 = 0;
		mmcdf999 = 0;
		mmcdf9999 = 0;
	}//end weibullstorage default constructor
};//end class weibullstorage
static vector<weibullstorage> weibullrawmax;//stores weibulls for raw maximum amplitudes
static vector<weibullstorage> weibulllpmax;//stores weibulls for lowpass maximum amplitudes
static vector<weibullstorage> weibullhpmax;//stores weibulls for highpass maximum amplitudes
static vector<weibullstorage> weibullrawmin;//stores weibulls for raw minimum amplitudes
static vector<weibullstorage> weibulllpmin;//stores weibulls for lowpass minimum amplitudes
static vector<weibullstorage> weibullhpmin;//stores weibulls for highpass minimum amplitudes
static vector<weibullstorage> weibullrawp2p;//stores weibulls for raw peak to peak amplitudes
static vector<weibullstorage> weibulllpp2p;//stores weibulls for lowpass peak to peak amplitudes
static vector<weibullstorage> weibullhpp2p;//stores weibulls for highpass peak to peak amplitudes
//global variables - configuration file
static vector<vector<float> > sncurves;//stores the sn curves for each channel for damage accumulation
static vector<float> channelscalefactors;//stores scale factors
static vector<float> channeloffsets;//stores offsets
static vector<string> channelnames;//stores channel names
static vector<int> performrainflowanalysis;//stores analysis channel choices
static vector<int> performmmmanalysis;//stores analysis channel choices
static vector<int> performpsdanalysis;//stores analysis channel choices
static vector<int> performraoanalysis;//stores analysis channel choices
static vector<int> performweibullanalysis;//stores analysis channel choices
static vector<float> emptyscalevalues;//stores the empty scale value (binary zero) for each channel
static vector<float> yellowlinevalues;//stores the yellow line value for each channel
static vector<float> redlinevalues;//stores the red line value for each channel
static vector<float> fullscalevalues;//stores the full scale value (binary max) for each channel
static vector<float> piersidezerovalues;//stores the pier size zero value for each channel
static vector<string> channelinformation;//stores extra channel information like gauge number
static vector<string> prescalefactorunits;//stores the units before the scale factor was applied
static vector<string> postscalefactorunits;//stores the units after the scale factor is applied
static vector<string> channellocations;//stores channel location information
static vector<int> virtualchanneltype;//stores the virtual channel type, 0 is no VC, of each possible VC
static vector<string> virtualchannelname;//stores the name of each virtual channel
static vector<float> virtualchannelprescalefactor;//stores the scale factor applied to real channels before combination
static vector<float> virtualchannelpostscalefactor;//stores the scale factor applied to virtual channels after combination
static vector<int> vctype1rect45rosette0degCH;//stores the channel number from a 0, 45, 90 rectangular rosette
static vector<float> vctype1rect45rosette0degOffset;//stores the offset from a 0, 45, 90 rectangular rosette
static vector<int> vctype1rect45rosette45degCH;//stores the channel number from a 0, 45, 90 rectangular rosette
static vector<float> vctype1rect45rosette45degOffset;//stores the offset from a 0, 45, 90 rectangular rosette
static vector<int> vctype1rect45rosette90degCH;//stores the channel number from a 0, 45, 90 rectangular rosette
static vector<float> vctype1rect45rosette90degOffset;//stores the offset from a 0, 45, 90 rectangular rosette
static vector<int> vctype1rect45rosetteOffsetIsMean;//stores the option to subtract each real channel's mean instead of the offset
static vector<int> vctype2add2CHA;//stores the channel number to add
static vector<int> vctype2add2CHB;//stores the channel number to add
static vector<int> vctype3sub2CHA;//stores the channel number to subtract
static vector<int> vctype3sub2CHB;//stores the channel number to subtract
static vector<int> vctype4avg2CHA;//stores the channel number to average
static vector<int> vctype4avg2CHB;//stores the channel number to average
static vector<int> vctype5suboffsetCH;//stores the channel number to subtract an offset from
static vector<float> vctype5suboffsetValue;//stores the offset to subtract
static vector<int> vctype6avg3CHA;//stores the channel number to average
static vector<int> vctype6avg3CHB;//stores the channel number to average
static vector<int> vctype6avg3CHC;//stores the channel number to average
//global variables - output flags
static vector<bool> zerovarianceflag;//stores a flag when the variance is zero
static vector<bool> emptyscaleflag;//stores a flag when the empty scale value is reached
static vector<bool> yellowlineflag;//stores a flag when the yellow line value is reached
static vector<bool> redlineflag;//stores a flag when the red line value is reached
static vector<bool> fullscaleflag;//stores a flag when the full scale value is reached
static vector<bool> hplpvarianceflag;//stores a flag when the highpass variance exceeds the lowpass variance
static vector<bool> hplpmaxflag;//stores a flag when the highpass max exceeds the lowpass max


//for conversion between high-byte-first and low-byte-first of 32-bit integers
static int swapbytes(int input){
	int output = 0;
	((char*)&output)[0] = ((char*)&input)[3];
	((char*)&output)[3] = ((char*)&input)[0];
	((char*)&output)[1] = ((char*)&input)[2];
	((char*)&output)[2] = ((char*)&input)[1];
	return output;
}//end swapbytes


//for conversion between high-byte-first and low-byte-first of 32-bit floating point number
static float swapbytes(float input){
	float output = 0;
	((char*)&output)[0] = ((char*)&input)[3];
	((char*)&output)[3] = ((char*)&input)[0];
	((char*)&output)[1] = ((char*)&input)[2];
	((char*)&output)[2] = ((char*)&input)[1];
	return output;
}//end swapbytes


//for conversion between high-byte-first and low-byte-first 64-bit floating point numbers
static double swapbytes(double input){
	double output = 0;
	((char*)&output)[0] = ((char*)&input)[7];
	((char*)&output)[7] = ((char*)&input)[0];
	((char*)&output)[1] = ((char*)&input)[6];
	((char*)&output)[6] = ((char*)&input)[1];
	((char*)&output)[2] = ((char*)&input)[5];
	((char*)&output)[5] = ((char*)&input)[2];
	((char*)&output)[3] = ((char*)&input)[4];
	((char*)&output)[4] = ((char*)&input)[3];
	return output;
}//end swapbytes


//this swap function is used in the fft function
static void SWAP(float& a, float& b){
	float temp = a;
	a = b;
	b = temp;
}//end SWAP


//these absolute value functions are used in the psd function
static int ABS(int a){
	if(a < 0) a = -a;
	return a;
}//end ABS
static float ABS(float a){
	if(a < 0) a = -a;
	return a;
}//end ABS
static double ABS(double a){
	if(a < 0) a = -a;
	return a;
}//end ABS


//This function loads a standard SCL binary data file
//as defined in "JHSV Analysis Engine" - NSWCCD-65-TR-2006/15.
static void loadSCLfile(string filename){
	ifstream inputfile;//the input filestream
	inputfile.open(filename.data(), ios::binary);//open the desired file in binary mode

	if(!inputfile.good()){//if the input file cannot be read
		cout << "Error: Cannot read input file " << filename << ".\n";
		cerr << "Error: Cannot read input file " << filename << ".\n";
		inputfile.close();
		exit(1);//exit completely with an error
	}//end if

	//read number of channels
	inputfile.read((char*)&numchan,4);//read the 4-byte integer
	numchan = swapbytes(numchan);//swap bytes from HBF to LBF
	realnumchan = numchan;//store the number of real data channels in a duplicate location
	if(numchan > 10000){//probable error
		cout << "Error: Numchan " << numchan << " > 10000 channels.\n";
		cerr << "Error: Numchan " << numchan << " > 10000 channels.\n";
		inputfile.close();
		exit(1);//exit completely with an error
	}//end if

	//read scanrate (model/recorded scale)
	inputfile.read((char*)&scanrate,4);//read the 4-byte float
	scanrate = swapbytes(scanrate);//swap bytes from HBF to LBF

	//read all cal factors
	cals.resize(numchan);//resize the cals vector/array to 1 for each channel
	for(int i=0; i<numchan; i++){
		inputfile.read((char*)&cal,8);//read the 8-byte doubles
		cal = swapbytes(cal);//swap bytes from HBF to LBF
		cals[i] = cal;//store 1 cal for each channel in cals vector
	}//end for

	//ready to read interleaved data at this point using numchan until end of file
	data.resize(numchan);//resize the data vector/array of arrays to 1 vector/array for each channel
	while(!inputfile.eof()){//continue until the end of file is reached
		for(int i=0; i<numchan && !inputfile.eof(); i++){//for each channel
			inputfile.read((char*)&datapt,4);//read the 4-byte float
			if(!inputfile.eof()){//if we are not at the end of the file
				datapt = swapbytes(datapt);//swap bytes from HBF to LBF
				data[i].insert(data[i].end(),datapt);//save the data point at the end of this channel's vector/array
			}//end if
		}//end for
	}//end while

	inputfile.close();//close the input file stream
}//end loadSCLfile


//This function resizes vectors after the number of channels is known or changed
static void resizeVectors(void){
	//data.resize(numchan);//occurs in loadSCLfile
	//cals.resize(numchan);//occurs in loadSCLfile
	datalp.resize(numchan);
	datahp.resize(numchan);

	extrema.resize(numchan);
	extremaindex.resize(numchan);
	rawcycles.resize(numchan);
	damage.resize(numchan);
	fullcyclehistograms.resize(numchan);

	means.resize(numchan);
	maxes.resize(numchan);
	mins.resize(numchan);
	variances.resize(numchan);
	skews.resize(numchan);
	kurtosi.resize(numchan);
	meanslp.resize(numchan);
	maxeslp.resize(numchan);
	minslp.resize(numchan);
	varianceslp.resize(numchan);
	skewslp.resize(numchan);
	kurtosilp.resize(numchan);
	meanshp.resize(numchan);
	maxeshp.resize(numchan);
	minshp.resize(numchan);
	varianceshp.resize(numchan);
	skewshp.resize(numchan);
	kurtosihp.resize(numchan);
	meanscycles.resize(numchan);
	maxescycles.resize(numchan);
	minscycles.resize(numchan);
	variancescycles.resize(numchan);
	skewscycles.resize(numchan);
	kurtosicycles.resize(numchan);

	psds.resize(numchan);
	psdpeakfreqs.resize(numchan);
	psd99pctfreqs.resize(numchan);
	raos.resize(numchan);

	weibullrawmax.resize(numchan);
	weibulllpmax.resize(numchan);
	weibullhpmax.resize(numchan);
	weibullrawmin.resize(numchan);
	weibulllpmin.resize(numchan);
	weibullhpmin.resize(numchan);
	weibullrawp2p.resize(numchan);
	weibulllpp2p.resize(numchan);
	weibullhpp2p.resize(numchan);

	sncurves.resize(numchan);
	channelscalefactors.resize(numchan);
	channeloffsets.resize(numchan);
	channelnames.resize(numchan);
	performrainflowanalysis.resize(numchan);
	performmmmanalysis.resize(numchan);
	performpsdanalysis.resize(numchan);
	performraoanalysis.resize(numchan);
	performweibullanalysis.resize(numchan);
	emptyscalevalues.resize(numchan);
	yellowlinevalues.resize(numchan);
	redlinevalues.resize(numchan);
	fullscalevalues.resize(numchan);
	piersidezerovalues.resize(numchan);
	channelinformation.resize(numchan);
	prescalefactorunits.resize(numchan);
	postscalefactorunits.resize(numchan);
	channellocations.resize(numchan);
	virtualchanneltype.resize(numchan);
	virtualchannelname.resize(numchan);
	virtualchannelprescalefactor.resize(numchan);
	virtualchannelpostscalefactor.resize(numchan);
	vctype1rect45rosette0degCH.resize(numchan);
	vctype1rect45rosette0degOffset.resize(numchan);
	vctype1rect45rosette45degCH.resize(numchan);
	vctype1rect45rosette45degOffset.resize(numchan);
	vctype1rect45rosette90degCH.resize(numchan);
	vctype1rect45rosette90degOffset.resize(numchan);
	vctype1rect45rosetteOffsetIsMean.resize(numchan);
	vctype2add2CHA.resize(numchan);
	vctype2add2CHB.resize(numchan);
	vctype3sub2CHA.resize(numchan);
	vctype3sub2CHB.resize(numchan);
	vctype4avg2CHA.resize(numchan);
	vctype4avg2CHB.resize(numchan);
	vctype5suboffsetCH.resize(numchan);
	vctype5suboffsetValue.resize(numchan);
	vctype6avg3CHA.resize(numchan);
	vctype6avg3CHB.resize(numchan);
	vctype6avg3CHC.resize(numchan);

	zerovarianceflag.resize(numchan);
	emptyscaleflag.resize(numchan);
	yellowlineflag.resize(numchan);
	redlineflag.resize(numchan);
	fullscaleflag.resize(numchan);
	hplpvarianceflag.resize(numchan);
	hplpmaxflag.resize(numchan);
}//end resizevectors


//this funciton reads a series of 1 variable for each channel in the config file
static void readConfigLineINT(ifstream& inputfile, vector<int>& v){
	string input;
	getline(inputfile,input,',');//get label
	for(unsigned int i=0; i < (unsigned)numchan; i++){
		if(i < (unsigned)numchan-1)getline(inputfile,input,',');
		else getline(inputfile,input,'\n');
		v[i] = atoi(input.data());
	}//end for
}//end readConfigLineINT


//this funciton reads a series of 1 variable for each channel in the config file
static void readConfigLineFLOAT(ifstream& inputfile, vector<float>& v){
	string input;
	getline(inputfile,input,',');//get label
	for(unsigned int i=0; i < (unsigned)numchan; i++){
		if(i < (unsigned)numchan-1)getline(inputfile,input,',');
		else getline(inputfile,input,'\n');
		v[i] = (float)atof(input.data());
	}//end for
}//end readConfigLineFLOAT


//this funciton reads a series of 1 variable for each channel in the config file
static void readConfigLineSTRING(ifstream& inputfile, vector<string>& v){
	string input;
	getline(inputfile,input,',');//get label
	for(unsigned int i=0; i < (unsigned)numchan; i++){
		if(i < (unsigned)numchan-1)getline(inputfile,input,',');
		else getline(inputfile,input,'\n');
		v[i] = input;
	}//end for
}//end readConfigLineSTRING


//this funciton reads a series of 1 variable for each channel in the config file
static void readConfigLineSNCURVE(ifstream& inputfile, vector< vector<float> >& v){
	string input;
	getline(inputfile,input,',');// get label
	for(unsigned int i=0; i < (unsigned)numchan; i++){
		if(i < (unsigned)numchan-1)getline(inputfile,input,',');
		else getline(inputfile,input,'\n');
		v[i].insert(sncurves[i].end(), (float)atof(input.data()));
	}//end for
}//end readConfigLineSNCURVE


//try to load configuration file
//autogen config files should be aeconfig-numchan-scanrate.csv
static void tryToLoadConfigFile(void){
	string input,configfilename;
	int tempscanrate = (int)scanrate;
	char temp[50];
	configfilename = "aeconfig-";
	sprintf(temp,"%i-%i.csv",numchan,tempscanrate);
	configfilename.append(temp);//finish building filename
	ifstream inputfile;//create an input filestream
	inputfile.open(configfilename.data(), ios::in);//try to open the config file

	if(!inputfile.good()){//if the config file could not be opened
		inputfile.close();//close it
		cout << "Warning: Cannot read config file " << configfilename << "; creating default one.\n";
		ofstream outputfile;//create an output filestream
		outputfile.open(configfilename.data(),ios::trunc);//open the config file for writing

		//start writing the default configuration file
		outputfile << "fftsize," << fftsize << endl;
		outputfile << "wavechannel," << wavechannel << endl;
		outputfile << "cutofffrequency," << cutofffrequency << endl;
		outputfile << "histogrambins," << bins << endl;
		outputfile << "modelscaleratio," << modelscaleratio << endl;
		outputfile << "numchan," << numchan << endl;
		outputfile << "scanrate," << scanrate << endl;

		outputfile << "channelnumber"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << i; outputfile << endl;
		outputfile << "name"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",ch" << i; outputfile << endl;
		outputfile << "scalefactor"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",1"; outputfile << endl;
		outputfile << "offset"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",0"; outputfile << endl;

		outputfile << "sn failure 1e0 cycle"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",320"; outputfile << endl;
		outputfile << "sn failure 1e1 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",300"; outputfile << endl;
		outputfile << "sn failure 1e2 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",260"; outputfile << endl;
		outputfile << "sn failure 1e3 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",200"; outputfile << endl;
		outputfile << "sn failure 1e4 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",150"; outputfile << endl;
		outputfile << "sn failure 1e5 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",120"; outputfile << endl;
		outputfile << "sn failure 1e6 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",90"; outputfile << endl;
		outputfile << "sn failure 1e7 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",80"; outputfile << endl;
		outputfile << "sn failure 1e8 cycles"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",70"; outputfile << endl;

		outputfile << "performrainflowanalysis"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",1"; outputfile << endl;
		outputfile << "performmmmanalysis"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",1"; outputfile << endl;
		outputfile << "performpsdanalysis"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",1"; outputfile << endl;
		outputfile << "performraoanalysis"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",1"; outputfile << endl;
		outputfile << "performweibullanalysis"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",1"; outputfile << endl;

		outputfile << "emptyscalevalue"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",-320"; outputfile << endl;
		outputfile << "yellowlinevalue"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",250"; outputfile << endl;
		outputfile << "redlinevalue"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",300"; outputfile << endl;
		outputfile << "fullscalevalue"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",400"; outputfile << endl;
		outputfile << "piersidezerovalue"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",50"; outputfile << endl;

		outputfile << "channelinformation"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",extra measurement info"; outputfile << endl;
		outputfile << "prescalefactorunits"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",Volts x Cal"; outputfile << endl;
		outputfile << "postscalefactorunits"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",Other EUs"; outputfile << endl;
		outputfile << "channellocation"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",frame 6 bulkhead 4"; outputfile << endl;

		outputfile << "virtualchanneltype"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "virtualchannelname"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << ",vc" << i; outputfile << endl;
		outputfile << "virtualchannelprescalefactor"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;
		outputfile << "virtualchannelpostscalefactor"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;

		outputfile << "vctype1rect45rosette0degCH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype1rect45rosette0degOffset"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype1rect45rosette45degCH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;
		outputfile << "vctype1rect45rosette45degOffset"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype1rect45rosette90degCH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 2; outputfile << endl;
		outputfile << "vctype1rect45rosette90degOffset"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype1rect45rosetteOffsetIsMean"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;

		outputfile << "vctype2add2CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype2add2CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;
		outputfile << "vctype3sub2CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype3sub2CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;
		outputfile << "vctype4avg2CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype4avg2CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;
		outputfile << "vctype5suboffsetCH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype5suboffsetValue"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype6avg3CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 0; outputfile << endl;
		outputfile << "vctype6avg3CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 1; outputfile << endl;
		outputfile << "vctype6avg3CH"; for(unsigned int i=0; i < (unsigned)numchan; i++) outputfile << "," << 2; outputfile << endl;

		outputfile.close();
		inputfile.clear();//clears any good or bad flags
		inputfile.open(configfilename.data(), ios::in);//open the newly written config file
	}//end if

	//start reading the configuration file

	//read single values
	getline(inputfile,input,','); getline(inputfile,input,'\n'); fftsize = atoi(input.data()); input.clear();
	getline(inputfile,input,','); getline(inputfile,input,'\n'); wavechannel = atoi(input.data()); input.clear();
	getline(inputfile,input,','); getline(inputfile,input,'\n'); cutofffrequency = (float)atof(input.data()); input.clear();
	getline(inputfile,input,','); getline(inputfile,input,'\n'); bins = atoi(input.data()); input.clear();
	getline(inputfile,input,','); getline(inputfile,input,'\n'); modelscaleratio = (float)atof(input.data()); input.clear(); fullscanrate = scanrate/sqrt(modelscaleratio);
	getline(inputfile,input,','); getline(inputfile,input,'\n'); confignumchan = atoi(input.data()); input.clear();
	getline(inputfile,input,','); getline(inputfile,input,'\n'); configscanrate = (float)atof(input.data()); input.clear();

	//channelnumbers line - skip
	getline(inputfile,input,'\n');

	//read lines with 1 variable per channel
	readConfigLineSTRING(inputfile, channelnames);
	readConfigLineFLOAT(inputfile, channelscalefactors);
	readConfigLineFLOAT(inputfile, channeloffsets);
	readConfigLineSNCURVE(inputfile, sncurves);//1e0
	readConfigLineSNCURVE(inputfile, sncurves);//1e1
	readConfigLineSNCURVE(inputfile, sncurves);//1e2
	readConfigLineSNCURVE(inputfile, sncurves);//1e3
	readConfigLineSNCURVE(inputfile, sncurves);//1e4
	readConfigLineSNCURVE(inputfile, sncurves);//1e5
	readConfigLineSNCURVE(inputfile, sncurves);//1e6
	readConfigLineSNCURVE(inputfile, sncurves);//1e7
	readConfigLineSNCURVE(inputfile, sncurves);//1e8
	readConfigLineINT(inputfile, performrainflowanalysis);
	readConfigLineINT(inputfile, performmmmanalysis);
	readConfigLineINT(inputfile, performpsdanalysis);
	readConfigLineINT(inputfile, performraoanalysis);
	readConfigLineINT(inputfile, performweibullanalysis);
	readConfigLineFLOAT(inputfile, emptyscalevalues);
	readConfigLineFLOAT(inputfile, yellowlinevalues);
	readConfigLineFLOAT(inputfile, redlinevalues);
	readConfigLineFLOAT(inputfile, fullscalevalues);
	readConfigLineFLOAT(inputfile, piersidezerovalues);
	readConfigLineSTRING(inputfile, channelinformation);
	readConfigLineSTRING(inputfile, prescalefactorunits);
	readConfigLineSTRING(inputfile, postscalefactorunits);
	readConfigLineSTRING(inputfile, channellocations);

	readConfigLineINT(inputfile, virtualchanneltype);
	readConfigLineSTRING(inputfile, virtualchannelname);
	readConfigLineFLOAT(inputfile, virtualchannelprescalefactor);
	readConfigLineFLOAT(inputfile, virtualchannelpostscalefactor);
	readConfigLineINT(inputfile, vctype1rect45rosette0degCH);
	readConfigLineFLOAT(inputfile, vctype1rect45rosette0degOffset);
	readConfigLineINT(inputfile, vctype1rect45rosette45degCH);
	readConfigLineFLOAT(inputfile, vctype1rect45rosette45degOffset);
	readConfigLineINT(inputfile, vctype1rect45rosette90degCH);
	readConfigLineFLOAT(inputfile, vctype1rect45rosette90degOffset);
	readConfigLineINT(inputfile, vctype1rect45rosetteOffsetIsMean);
	readConfigLineINT(inputfile, vctype2add2CHA);
	readConfigLineINT(inputfile, vctype2add2CHB);
	readConfigLineINT(inputfile, vctype3sub2CHA);
	readConfigLineINT(inputfile, vctype3sub2CHB);
	readConfigLineINT(inputfile, vctype4avg2CHA);
	readConfigLineINT(inputfile, vctype4avg2CHB);
	readConfigLineINT(inputfile, vctype5suboffsetCH);
	readConfigLineFLOAT(inputfile, vctype5suboffsetValue);
	readConfigLineINT(inputfile, vctype6avg3CHA);
	readConfigLineINT(inputfile, vctype6avg3CHB);
	readConfigLineINT(inputfile, vctype6avg3CHC);

	inputfile.close();

	//determine number of virtual channels
	for(int i=0; i < realnumchan; i++)
		if(virtualchanneltype[i] > 0)
			virtualnumchan++;

}//end tryToLoadConfigFile


//this function applies scale factors and offsets read from
//the configfuration file to the original raw data before
//any analysis has been performed
static void applysfo(void){
	float scalefactor,offset;
	for(unsigned int i=0; i < (unsigned)numchan; i++){
		scalefactor = channelscalefactors[i];
		offset = channeloffsets[i];
		if(offset != 0) for(unsigned int j=0; j < data[i].size(); j++) data[i][j] -= offset;
		if(scalefactor != 1) for(unsigned int j=0; j < data[i].size(); j++) data[i][j] *= scalefactor;
		cout << "."; cout.flush();
	}//end for
}//end applysfo


//compute virtual channels
static void computevirtualchannels(void){
	if(virtualnumchan == 0) return;//bail out if there are no virtual channels
	numchan = realnumchan + virtualnumchan;//update numchan
	resizeVectors();//update all vectors to accomodate additional channels

	//resize remaining vectors
	data.resize(numchan);//make room for virtual channels - not handled by resizeVectors
	cals.resize(numchan);//make room for virtual channel cals - not handled by resizeVectors

	//duplicate configuration values
	//this allows the virtual channels to be handled as if they
	//existed in the original data file, which should be the least amount of code changes overall
	//only a couple minor changes on the output reading side should be necessary
	int tempvirtualnumchan = 0;
	for(int i=0; i < realnumchan; i++){
		if(virtualchanneltype[i] > 0){
			cals[realnumchan+tempvirtualnumchan] = cals[i];
			sncurves[realnumchan+tempvirtualnumchan] = sncurves[i];
			channelscalefactors[realnumchan+tempvirtualnumchan] = channelscalefactors[i];
			channeloffsets[realnumchan+tempvirtualnumchan] = channeloffsets[i];
			channelnames[realnumchan+tempvirtualnumchan] = virtualchannelname[i];
			performrainflowanalysis[realnumchan+tempvirtualnumchan] = performrainflowanalysis[i];
			performmmmanalysis[realnumchan+tempvirtualnumchan] = performmmmanalysis[i];
			performpsdanalysis[realnumchan+tempvirtualnumchan] = performpsdanalysis[i];
			performraoanalysis[realnumchan+tempvirtualnumchan] = performraoanalysis[i];
			performweibullanalysis[realnumchan+tempvirtualnumchan] = performweibullanalysis[i];
			emptyscalevalues[realnumchan+tempvirtualnumchan] = emptyscalevalues[i];
			yellowlinevalues[realnumchan+tempvirtualnumchan] = yellowlinevalues[i];
			redlinevalues[realnumchan+tempvirtualnumchan] = redlinevalues[i];
			fullscalevalues[realnumchan+tempvirtualnumchan] = fullscalevalues[i];
			piersidezerovalues[realnumchan+tempvirtualnumchan] = piersidezerovalues[i];
			channelinformation[realnumchan+tempvirtualnumchan] = channelinformation[i];
			prescalefactorunits[realnumchan+tempvirtualnumchan] = prescalefactorunits[i];
			postscalefactorunits[realnumchan+tempvirtualnumchan] = postscalefactorunits[i];
			channellocations[realnumchan+tempvirtualnumchan] = channellocations[i];

			virtualchanneltype[realnumchan+tempvirtualnumchan] = virtualchanneltype[i];
			virtualchannelname[realnumchan+tempvirtualnumchan] = virtualchannelname[i];
			virtualchannelprescalefactor[realnumchan+tempvirtualnumchan] = virtualchannelprescalefactor[i];
			virtualchannelpostscalefactor[realnumchan+tempvirtualnumchan] = virtualchannelpostscalefactor[i];
			vctype1rect45rosette0degCH[realnumchan+tempvirtualnumchan] = vctype1rect45rosette0degCH[i];
			vctype1rect45rosette0degOffset[realnumchan+tempvirtualnumchan] = vctype1rect45rosette0degOffset[i];
			vctype1rect45rosette45degCH[realnumchan+tempvirtualnumchan] = vctype1rect45rosette45degCH[i];
			vctype1rect45rosette45degOffset[realnumchan+tempvirtualnumchan] = vctype1rect45rosette45degOffset[i];
			vctype1rect45rosette90degCH[realnumchan+tempvirtualnumchan] = vctype1rect45rosette90degCH[i];
			vctype1rect45rosette90degOffset[realnumchan+tempvirtualnumchan] = vctype1rect45rosette90degOffset[i];
			vctype1rect45rosetteOffsetIsMean[realnumchan+tempvirtualnumchan] = vctype1rect45rosetteOffsetIsMean[i];
			vctype2add2CHA[realnumchan+tempvirtualnumchan] = vctype2add2CHA[i];
			vctype2add2CHB[realnumchan+tempvirtualnumchan] = vctype2add2CHB[i];
			vctype3sub2CHA[realnumchan+tempvirtualnumchan] = vctype3sub2CHA[i];
			vctype3sub2CHB[realnumchan+tempvirtualnumchan] = vctype3sub2CHB[i];
			vctype4avg2CHA[realnumchan+tempvirtualnumchan] = vctype4avg2CHA[i];
			vctype4avg2CHB[realnumchan+tempvirtualnumchan] = vctype4avg2CHB[i];
			vctype5suboffsetCH[realnumchan+tempvirtualnumchan] = vctype5suboffsetCH[i];
			vctype5suboffsetValue[realnumchan+tempvirtualnumchan] = vctype5suboffsetValue[i];
			vctype6avg3CHA[realnumchan+tempvirtualnumchan] = vctype6avg3CHA[i];
			vctype6avg3CHB[realnumchan+tempvirtualnumchan] = vctype6avg3CHB[i];
			vctype6avg3CHC[realnumchan+tempvirtualnumchan] = vctype6avg3CHC[i];

			tempvirtualnumchan++;
		}//end if
	}//end for

	//call function to compute channels and duplicate config info for
	float sqrt2 = sqrt((float)2.0);
	for(int i=0; i < virtualnumchan; i++){
		//make sure there are data points
		data[realnumchan+i].resize(data[0].size());
		float presf = virtualchannelprescalefactor[realnumchan+i];
		float postsf = virtualchannelpostscalefactor[realnumchan+i];

		//compute rosette rectangular 0,45,90
		if(virtualchanneltype[realnumchan+i] == 1){
			//http://www.vishay.com/docs/11065/_vmr-tc0.pdf page 5 eq 3
			double mean1=0;
			double mean2=0;
			double mean3=0;
			if(vctype1rect45rosetteOffsetIsMean[realnumchan+i]){
				for(unsigned int m=0; m < data[0].size(); m++){
					mean1 += data[vctype1rect45rosette0degCH[realnumchan+i]][m];
					mean2 += data[vctype1rect45rosette45degCH[realnumchan+i]][m];
					mean3 += data[vctype1rect45rosette90degCH[realnumchan+i]][m];
				}//end for
				mean1/=data[0].size();
				mean2/=data[0].size();
				mean3/=data[0].size();
			}//end if
			for(unsigned int j=0; j < data[0].size(); j++){
				float e1 = data[vctype1rect45rosette0degCH[realnumchan+i]][j];
				float e2 = data[vctype1rect45rosette45degCH[realnumchan+i]][j];
				float e3 = data[vctype1rect45rosette90degCH[realnumchan+i]][j];
				if(vctype1rect45rosetteOffsetIsMean[realnumchan+i]){
					e1 -= (float)mean1;
					e2 -= (float)mean2;
					e3 -= (float)mean3;
				}//end if
				else{
					e1 -= vctype1rect45rosette0degOffset[realnumchan+i];
					e2 -= vctype1rect45rosette45degOffset[realnumchan+i];
					e3 -= vctype1rect45rosette90degOffset[realnumchan+i];
				}//end else
				e1 *= presf;
				e2 *= presf;
				e3 *= presf;
				data[realnumchan+i][j] = (e1+e3)/2 + 1/sqrt2 * sqrt( (e1-e2)*(e1-e2) + (e2-e3)*(e2-e3) );
				data[realnumchan+i][j] *= postsf;
			}//end for
		}//end if
		//add 2 channels
		else if(virtualchanneltype[realnumchan+i] == 2){
			for(unsigned int j=0; j < data[0].size(); j++){
				data[realnumchan+i][j] = presf * data[vctype2add2CHA[realnumchan+i]][j] + presf * data[vctype2add2CHB[realnumchan+i]][j];
				data[realnumchan+i][j] *= postsf;
			}//end for
		}//end if
		//subtract 2 channels
		else if(virtualchanneltype[realnumchan+i] == 3){
			for(unsigned int j=0; j < data[0].size(); j++){
				data[realnumchan+i][j] = presf * data[vctype3sub2CHA[realnumchan+i]][j] + presf * data[vctype3sub2CHB[realnumchan+i]][j];
				data[realnumchan+i][j] *= postsf;
			}//end for
		}//end if
		//avgerage 2 channels
		else if(virtualchanneltype[realnumchan+i] == 4){
			for(unsigned int j=0; j < data[0].size(); j++){
				data[realnumchan+i][j] = (presf * data[vctype4avg2CHA[realnumchan+i]][j] + presf * data[vctype4avg2CHB[realnumchan+i]][j])/2;
				data[realnumchan+i][j] *= postsf;
			}//end for
		}//end if
		//subtract an offset from a channel
		else if(virtualchanneltype[realnumchan+i] == 5){
			for(unsigned int j=0; j < data[0].size(); j++){
				data[realnumchan+i][j] = presf * data[vctype5suboffsetCH[realnumchan+i]][j] - vctype5suboffsetValue[realnumchan+i];
				data[realnumchan+i][j] *= postsf;
			}//end for
		}//end if
		//average 3 channels
		else if(virtualchanneltype[realnumchan+i] == 6){
			for(unsigned int j=0; j < data[0].size(); j++){
				data[realnumchan+i][j] = (presf * data[vctype6avg3CHA[realnumchan+i]][j] + presf * data[vctype6avg3CHB[realnumchan+i]][j] + presf * data[vctype6avg3CHC[realnumchan+i]][j])/3;
				data[realnumchan+i][j] *= postsf;
			}//end for
		}//end if
		
		cout << "."; cout.flush();
	}//end for
}//end computevirtualchannels


//computes mmm stats for a given channel
static void mmmstats(vector<float>&data,double&mean,float&max,float&min,double&variance,double&skew,double&kurtosis){
	mean = 0;
	max = 0;
	min = 0;
	variance = 0;
	skew = 0;
	kurtosis = 0;

	if(data.size() <= 1){
		return;
	}//end if

	max = data[0];
	min = data[0];

	//first moment
	for(unsigned int i=0; i < data.size(); i++){
		mean += data[i];
		if(data[i] > max)
			max = data[i];
		else if(data[i] < min)
			min = data[i];
	}//end for
	mean /= data.size();

	//second central moment; mean is the center
	for(unsigned int i=0; i < data.size(); i++){
		variance += pow(data[i] - mean,2);
	}//end for
	variance /= (data.size()-1);
	double stdev = pow(variance,0.5);

	//third and fourth normalized central moments; mean is the center; normalized by /sigma^n
	for(unsigned int i=0; i < data.size() && variance != 0; i++){
		skew += pow((data[i] - mean),3);
		kurtosis += pow((data[i] - mean),4);
	}//end for
	skew /= data.size() * pow(stdev,3);
	kurtosis /= data.size() * pow(stdev,4);

}//end mmmstats


//computes flags for a given channel
//should be run after all mmm analysis
static void computeflags(int channel){
	if(variances[channel] == 0) zerovarianceflag[channel] = 1;
	if(mins[channel] <= emptyscalevalues[channel]) emptyscaleflag[channel] = 1;
	if(maxes[channel] >= yellowlinevalues[channel]) yellowlineflag[channel] = 1;
	if(maxes[channel] >= redlinevalues[channel]) redlineflag[channel] = 1;
	if(maxes[channel] >= fullscalevalues[channel]) fullscaleflag[channel] = 1;
	if(varianceshp[channel] > varianceslp[channel]) hplpvarianceflag[channel] = 1;
	if(maxeshp[channel]-meanshp[channel] > maxeslp[channel]-meanslp[channel]) hplpmaxflag[channel] = 1;
}//end computeflags


//This sorting method is recursive quicksort, from
//http://en.wikipedia.org/wiki/Quicksort
static void quicksort(vector<float>& data){
	if(data.size() <= 1) return;
	vector<float> less, pivotlist, greater;
	float pivot = data[0];
	for(unsigned int i=1; i < data.size(); i++){
		if(data[i] < pivot) less.insert(less.end(),data[i]);
		else greater.insert(greater.end(),data[i]);
	}//end for
	pivotlist.insert(pivotlist.end(),pivot);
	quicksort(less);
	quicksort(greater);
	data.resize(0);
	for(unsigned int i=0; i < less.size(); i++)data.insert(data.end(),less[i]);
	for(unsigned int i=0; i < pivotlist.size(); i++)data.insert(data.end(),pivotlist[i]);
	for(unsigned int i=0; i < greater.size(); i++)data.insert(data.end(),greater[i]);
}//end quicksort


//this function uses the first derivative to find
//every oscillation in the time history
static void selectrainflowextrema(int channel){

	extrema[channel].resize(0);
	extremaindex[channel].resize(0);
	for(unsigned int i=1; i < data[channel].size()-1; i++){
		if((data[channel][i-1] >= data[channel][i] && data[channel][i+1] > data[channel][i]) || (data[channel][i-1] <= data[channel][i] && data[channel][i+1] < data[channel][i])){
			extrema[channel].insert(extrema[channel].end(), data[channel][i]);
			extremaindex[channel].insert(extremaindex[channel].end(),i);
		}//end if
	}//end for

} //rainflow extrema





//this function uses rainflow algorithm 1 from
//Simple rainflow counting algorithms   ARTICLE
//International Journal of Fatigue, Volume 4, Issue 1, January 1982, Pages 31-40
//S. D. Downing and D. F. Socie
static void performrainflow(int channel){

	tempextrema.resize(0);
	rawcycles[channel].resize(0);

	if(extrema[channel].size() >= 3){

		//rearrange extrema to begin and end with maximum peak
		float max;
		unsigned int maxloc;
		max = extrema[channel][0];
		maxloc = 0;
		for(unsigned int i=0; i < extrema[channel].size(); i++){
			if(extrema[channel][i] > max){
				max = extrema[channel][i];
				maxloc = i;
			}//end if
		}//end for
		for(unsigned int i=maxloc; i < extrema[channel].size(); i++){
			tempextrema.insert(tempextrema.end(),extrema[channel][i]);
		}//end for
		for(unsigned int i=0; i <= maxloc; i++){
			tempextrema.insert(tempextrema.end(),extrema[channel][i]);
		}//end for

		//start rainflow algorithm 1 from D&S
		float x, y;
		int step=1;
		tempcycles.resize(0);
		unsigned int i=0;
		do{
			//step 1
			if(step == 1){
				if(i < tempextrema.size()){
					//get next extremum
					tempcycles.insert(tempcycles.end(),tempextrema[i]);
					i=i+1;
					step = 2;
				}//end if
				else{
					step = 5;
				}//end else
			}//end if

			//step 2
			if(step == 2){
				if(tempcycles.size() < 3){
					step = 1;
				}//end if
				else{
					//determine ranges x and y
					y = ABS(tempcycles[tempcycles.size()-3] - tempcycles[tempcycles.size()-2]);
					x = ABS(tempcycles[tempcycles.size()-2] - tempcycles[tempcycles.size()-1]);
					if(x == 0 || y == 0){
						tempcycles.erase(tempcycles.begin()+tempcycles.size()-2);
						step = 2;
					}//end if
					else{
						step = 3;
					}//end else
				}//end else
			}//end if

			//step 3
			if(step == 3){
				if( x < y){
					step = 1;
				}//end if
				else{
					step = 4;
				}//end else
			}//end if

			//step 4
			if(step == 4){
				//store cycle and erase extrema
				rawcycles[channel].insert(rawcycles[channel].end(),y);
				tempcycles.erase(tempcycles.begin()+tempcycles.size()-3);
				tempcycles.erase(tempcycles.begin()+tempcycles.size()-2);
				step = 2;
			}//end if

		}while(step != 5);//end do-while
	}//end if
} //perform rainflow






#endif

